from nwtoolkit.utilities.environment import hosts
import nwtoolkit.utilities.color as color
import os
import hashlib
import pathlib

def entry(args):
    sync_files = []
    for root, directories, files in os.walk(args[1]):
        for f in files:
            sync_files.append(
                os.path.join(root, f)
            )
    
    out_of_sync = []
    for filename in sync_files:
        with open(filename, 'rb') as f:
            correct_hash = hashlib.md5(f.read()).hexdigest()
        not_synced = checkFile(correct_hash, filename.replace(args[1], ''))
        if len(not_synced):
            print(color.red('''#########################
# This is where we'd send an email
##########################'''))
            sync = (input('Would you like to sync the file now? (y/n): ') == 'y')
            if sync:
                for host in not_synced:
                    sendFile(host, filename, filename.replace(args[1], ''))

def sendFile(host, path, shortpath):
    fullpath = os.path.join(pathlib.Path.cwd(), path)
    host._head.sendFile(fullpath, fullpath.split('\\')[-1])
    cmd = 'salt-cp ' + host.minionid + ' /tmp/' + fullpath.split('\\')[-1]+ ' ' + shortpath
    host._head.sendCommand(cmd.replace('\\', '/'))
    print(color.green(host.hostname))

def checkFile(correct_hash, path):
    path = path.replace('\\', '/')
    print(f'Checking {path} for consistency with hash {correct_hash}')
    not_synced = []
    for salt_id, host in hosts.items():
        stdin, stdout, stderr = host.sendCommand(f'md5sum {path}')
        for line in stdout:
            if salt_id in line:
                continue
            file_hash = line.split('/')[0].replace(' ', '')
            if file_hash.lower() == correct_hash.lower():
                print(color.green(f'{host.hostname} is in sync'))
            else:
                print(color.red(f'{host.hostname} is not in sync'))
                not_synced.append(host)
    return not_synced
